from django.conf import settings
from django.db import models
from django.contrib.auth.models import UserManager, User
from django.utils.translation import ugettext_lazy as _
from webutils.djtools.email import send_simple_email


class BaseUserProfileManager(UserManager):
    def generate_hash(self):
        import hashlib
        import random

        salt = hashlib.sha1(str(random.random())).hexdigest()[:5]
        random_string = User.objects.make_random_password()
        return hashlib.sha1(salt + random_string).hexdigest()

    def send_user_email(self, user, subject, context, template):
        send_simple_email(
            user.email,
            subject,
            context,
            template,
            is_template_file=True,
        )

    def add_new_user(self, username, password, email, first_name, last_name):
        new_user = User.objects.create_user(username, email, password)
        new_user.first_name = first_name
        new_user.last_name = last_name
        new_user.is_active = True
        new_user.save()

        # User Profile
        return self.create(user=new_user)

    def reset_user_password(self, key):
        import re

        if re.match('^[a-f0-9]{40}$', key):
            try:
                profile = self.get(activation_key=key)
            except self.model.DoesNotExist:
                return False

            user = profile.user
            new_password = User.objects.make_random_password(length=8)
            user.set_password(new_password)
            user.save()

            profile.activation_key = ''
            profile.save()

            subject = 'Your Password Reset Request'
            ctext = {
                'password': new_password,
            }
            self.send_user_email(
                user,
                subject,
                ctext,
                'baseacct/reset_email_done.txt'
            )
            return user
        return False


class BaseUserProfile(models.Model):
    user = models.ForeignKey(
        User,
        unique=True,
        verbose_name=_('User Account'),
        help_text=_('User Profiles must be "tied" to a user account.'),
        related_name='%(app_label)s_%(class)s_set',
    )
    
    # Activation Key
    activation_key = models.CharField(
        max_length=125,
        blank=True,
        editable=False,
    )

    # Custom Manager
    objects = BaseUserProfileManager()

    # Metadata
    class Meta:
        abstract = True
        ordering = ('user',)

    # Functions
    def __unicode__(self):
        return u'%s' % (self.user.username)



import hmac
import time
import base64
import urllib
import hashlib
import urlparse


class SecureS3(object):
    def __init__(self, key, secret_key):
        self.key = key
        self.secret_key = secret_key

    def gen_signature(self, string_to_sign):
        return base64.encodestring(
            hmac.new(
                self.secret_key,
                string_to_sign,
                hashlib.sha1
            ).digest()
        ).strip()

    def get_file_details(self, url):
        ''' Returns bucket name and file from an S3 URL
        '''
        amazon_host = 's3.amazonaws.com'
        s = urlparse.urlparse(url)
        if not s.path or not s.path[1:]:
            raise ValueError('Invalid S3 file passed.')
			
        if s.netloc == amazon_host:
            # s3.amazonaws.com/bucket/file...
            bucket = s.path[1:].split('/')[0]
            filename = '/'.join(s.path[1:].split('/')[1:])
        elif s.netloc.endswith('.%s' % amazon_host):
            # bucket_name.s3.amazonaws.com/file...
            bucket = s.netloc.replace('.%s' % amazon_host, '')
            filename = s.path[1:]
        else:
            # bucket.com/file... (CNAME BUCKET URL)
            bucket = s.netloc
            filename = s.path[1:]

        return (bucket, filename, s.scheme)

    def get_auth_link(self, bucket, filename, 
                      scheme='http', expires=300, timestamp=None):
        ''' Return a secure S3 link with an expiration on the download.

            key: S3 Access Key (login)
            secret_key: S3 Secret Access Key (password)
            bucket: Bucket name
            filename: file path
            expires: Seconds from NOW the link expires
            timestamp: Epoch timestamp. If present, "expires" will not be used.
        '''
        filename = urllib.quote_plus(filename)
        filename = filename.replace('%2F', '/')
        path = '/%s/%s' % (bucket, filename)

        if timestamp is not None:
            expire_time = float(timestamp)
        else:
            expire_time = time.time() + expires

        expire_str = '%.0f' % (expire_time)
        string_to_sign = u'GET\n\n\n%s\n%s' % (expire_str, path)
        params = {
            'AWSAccessKeyId': self.key,
            'Expires': expire_str,
            'Signature': self.gen_signature(string_to_sign.encode('utf-8')),
        }

        return '%s://s3.amazonaws.com/%s/%s?%s' % (
                        scheme, bucket, filename, urllib.urlencode(params))
    
    def get_easy_auth_link(self, url, expires=600):
        ''' url should be the full URL to the secure file hosted on S3.
            examples:
            http://s3.amazonaws.com/your-bucket/yourfile.zip
            http://your-bucket.s3.amazonaws.com/yourfile.zip
            http://media.your-domain.com/yourfile.zip  (CNAME path to S3)
        '''
        try:
            data = self.get_file_details(url)
        except ValueError:
            return None

        return self.get_auth_link(*data[:2], scheme=data[2], expires=expires)


from django.conf import settings
from django.core.mail import send_mail
from django.template import Context, Template, loader


def send_simple_email(recip, subject, context, template,
                from_email=None, is_template_file=True, fail_silently=False):
    if is_template_file:
        # Load template from actual file in templates dir
        msg_template = loader.get_template(template)
    else:
        # Template is a string, passed directly
        msg_template = Template(template)

    msg_context = Context(context)
    msg_dict = {
        'from_email': from_email or settings.DEFAULT_FROM_EMAIL,
        'message': msg_template.render(msg_context),
        'recipient_list': isinstance(recip, list) and recip or [recip],
        'subject': subject,
    }
    send_mail(fail_silently=fail_silently, **msg_dict)



''' General helper functions that are useful to reuse.
'''


def easy_encode(value, enc_type='utf-8'):
    ''' Try to .encode(enc_type) on value.
    '''
    try:
        return value.encode(enc_type)
    except AttributeError:
        return value


def encode_dict(data, enc_type='utf-8'):
    ''' Take a dictionary and encode appropriate variables 
        to specified encoding.
    '''
    for key, value in data.items():
        data[key] = easy_encode(value, enc_type)
    return data


def grab_from_import(module, as_from=False):
    ''' Used to import a object, class, etc. from a Python module. 
    
        Should be used when you want to do the equivilent of
         
            from mymodule import someclass
        
        So you would pass in as the "module" variable:
        
            grab_from_import('mymodule.someclass', as_from=True)
        
        If you simply wanted to do the following:
        
            import mymodule.someclass
        
        You just pass in:
        
            grab_from_import('mymodule.someclass')
        
        The 'default_module' variable will be returned if the 
    '''
    if not as_from:
        return __import__(module, globals(), locals(), [], -1)
    else:
        _pieces = module.split('.')
        _module = '.'.join(_pieces[:-1])
        _obj = _pieces[-1]

        _temp = __import__(_module, globals(), locals(), [_obj,], -1)
        return getattr(_temp, _obj)


def map_translate(data, source, re_map=r'{([\w-]+)}', 
                                src_map='{%s}', allowed_fields=[]):
    ''' Easily create templatized variables for re-use.
        
        By default it will replace any instance of 
        
            {variable_name}
        
        in 'data' with 'source'.variable_name. If 'source' is 
        a dictionary, it will use variable_name as the dictionary 
        key to access the data. Otherwise we will use getattr to 
        grab the data. Accepts callable methods to return data.
        
        data = Data that has template variables
        source = Dictionary or Class type object that contains data 
                 to replace template variables.
        re_map = regular expression to use when matching template 
                 variables
        src_map = Used for actually doing the string replace in the 
                  data variable.
        allowed_fields = List of allowed fields. If specified, then 
                         only fields listed can be templatized.
    '''
    import re
    import copy
    
    def get_translation(source, field):
        if isinstance(source, dict):
            ret = source.get(field, None)
        else:
            ret = getattr(source, field, None)

        if ret is not None and callable(ret):
            ret = ret()
        return str(ret)
    
    new_data = copy.copy(data)
    for field in re.findall(re_map, new_data):
        if allowed_fields and field not in allowed_fields:
            continue
            
        new_data = new_data.replace(
            src_map % field,
            get_translation(source, field),
        )
    return new_data


def strip_unwanted(data, allowed_chars='0123456789'):
    ''' Take "data" variable and strip out every character
        that isn't in "allowed_chars" variable
    '''
    return ''.join([x for x in data if x in allowed_chars])


import hashlib

def thankyou_check(key, data):
    '''
        Verifies hash passed via clickbank site to 
        thank you page.
        https://www.clickbank.com/publisher_tools.html#Publisher_Tools_7
    '''
    if not isinstance(data, dict):
        return False

    try:
        rcpt = data['cbreceipt']
        time = data['time']
        item = data['item']
        cbpop = data['cbpop']
    except KeyError:
        return False

    hash_str = '%s|%s|%s|%s' % (key, rcpt, time, item)
    ckpop = hashlib.sha1(hash_str).hexdigest()[:8].upper()
    return ckpop == cbpop


def ipncheck(key, data):
    '''
        Verifies hash passed via clickbank IPN notification.
        https://www.clickbank.com/20080219_release_summary.html
    '''
    vars = (
        'ccustname',
        'ccustemail',
        'ccustcc',
        'ccuststate',
        'ctransreceipt',
        'cproditem',
        'ctransaction',
        'ctransaffiliate',
        'ctranspublisher',
        'cprodtype',
        'cprodtitle',
        'ctranspaymentmethod',
        'ctransamount',
        'caffitid',
        'cvendthru',
    )

    if not isinstance(data, dict):
        return False

    hash_str = ''
    for var in vars:
        try:
            hash_str += '%s|' % (data[var])
        except KeyError:
            return False

    hash_str += '%s' % (key)
    cbpop = data['cverify']
    ckpop = hashlib.sha1(hash_str).hexdigest()[:8].upper()
    return ckpop == cbpop



''' Copyright 2009 Peter Sanchez <petersanchez@gmail.com>.  
    See BSD-LICENSE for license information.
    
    Parse / Convert XML data.
    
    Example:
    
    {'sales': [
        {'sale': 
            {'id': 1, 'name': 'test1', 'null-field': None}},
        {'sale': 
            {'id': 2, 'name': 'test2'}}
    ]}
    
    Becomes:
    
    <sales type="array">
        <sale>
            <id type="integer">1</id>
            <name>test1</name>
            <null-field nil="true" />
        </sale>
        <sale>
            <id type="integer">2</id>
            <name>test2</name>
        </sale>
    </sales>
    
    And vise versa.
    
    Call Parser().build_request_string() to convert a dictionary to XML.
    
    Call Parser().parse() to convert an XML string to a dictionary.
'''
import xml.etree.cElementTree as ET


class ParserError(Exception):
    pass
    

class Parser(object):
    def get_key(self, element, key, default=None):
        return element.get(key, default)
    
    def is_nil(self, element):
        return (self.get_key(element, 'nil', 'false') == 'true')
    
    def get_type(self, element):
        return self.get_key(element, 'type', 'string')
    
    def get_data(self, element):
        ret = None
        if not self.is_nil(element):
            etype = self.get_type(element)
            if etype == 'integer':
                ret = int(element.text)
            elif etype == 'boolean':
                ret = (element.text == 'true')
            elif etype == 'array':
                ret = [{
                    c.tag: self.parse_children(c),
                } for c in element.getchildren()]
            else:
                if not element.getchildren():
                    ret = element.text
                else:
                    ret = self.parse_children(element)
        return ret
        
    def parse(self, data):
        ret = {}
        try:
            root = ET.XML(data)
        except SyntaxError, err:
            raise ParserError('Invalid XML data given.')
        ret[root.tag] = self.get_data(root)
        return ret

    def parse_children(self, elements):
        ret = {}
        for child in elements:
            ret[child.tag] = self.get_data(child)
        return ret
    
    def parse_errors(self, data):
        try:
            root = ET.XML(data)
        except SyntaxError, err:
            raise ParserError('Invalid XML data given.')
        return [e.text for e in root.findall('error')]
    
    def parse_request(self, zhash, element=None):
        if element is None:
            if len(zhash) > 1:
                raise ParserError('Invalid data dictionary given.')
            key = zhash.keys()[0]
            data = zhash[key]
            element = ET.Element(key)
            self.parse_request(data, element)
            return element

        else:
            etype = None
            if isinstance(zhash, (list, tuple)):
                etype = 'array'
                for item in zhash:
                    self.parse_request(item, element)
            elif isinstance(zhash, dict):
                for key, value in zhash.iteritems():
                    sub = ET.SubElement(element, key)
                    self.parse_request(value, sub)
            elif isinstance(zhash, bool):
                etype = 'boolean'
                element.text = 'true' if zhash else 'false'
            elif isinstance(zhash, int):
                etype = 'integer'
                element.text = str(zhash)
            elif zhash is None:
                element.set('nil', 'true')
            else:
                element.text = str(zhash)
            
            if etype is not None:
                element.set('type', etype)

    def build_request_string(self, zhash):
        return '<?xml version="1.0" encoding="UTF-8"?>%s' % \
                                        ET.tostring(self.parse_request(zhash))



'''
    Class to offer a "One Time Only" Offer for your 
    sales pages.

    This will allow you to offer pages on a "one time only" 
    basis. It will track the current position based on a 
    cookie that is named after your project name (taken 
    from settings.ROOT_URLCONF).

    Initiate the instance with a tuple, or list, of OTO templates. 
    In other words, how many "one time offers" you want to go 
    through before finally settling on the "final_template".

    It also accepts a variable named "cookie_days" which is a setting 
    for how many days the cookie should live on the users browser.

    Example usage in urls.py..

    from yourproject.OTO import OTO

    oto_templates = ('template1.html', 'template2.html', 'template3.html')
    urlpatterns = patterns('',
        (r'^oto/$',
            OTO(oto_templates, 'final_template.html')),
    )

    Head to yourdomain.com/oto you will see template1.html rendered. Reload 
    for template2.html, template3.html and eventually final_template.html

    You can set settings.OTO_COOKIE_DOMAIN to manually set the domain 
    that the cookie is set for. Otherwise it checks the Site framework and 
    if that fails, uses request.get_host() (requires Django 1.0+!)

    Peter Sanchez
    pjs@petersanchez.com - www.petersanchez.com
'''
from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.sites.models import Site


class OTO(object):
    def __init__(self, oto_templates, final_template, cookie_days=180):
        if not isinstance(oto_templates, (list, tuple)):
            oto_templates = (oto_templates,)

        self.oto_templates = oto_templates
        self.final_template = final_template
        self.cookie_days = cookie_days

        self.cookie_str = '%s-oto' % (settings.ROOT_URLCONF.split('.')[0])
        self.cookie_domain = None

    def _getvars(self, request):
        getvars = request.GET.copy()
        if len(getvars.keys()) > 0:
            return '&%s' % getvars.urlencode()
        return ''

    def return_response(self, request, template):
        return render_to_response(template, {
            'cookie_domain': self.cookie_domain,
            'getvars': self._getvars(request),
            }, context_instance=RequestContext(request))

    def get_cookie_domain(self, request):
        if hasattr(settings, 'OTO_COOKIE_DOMAIN'):
            self.cookie_domain = settings.OTO_COOKIE_DOMAIN
        else:
            try:
                site = Site.objects.get_current()
                self.cookie_domain = site.domain
            except Site.DoesNotExist:
                self.cookie_domain = request.get_host()

    def __call__(self, request):
        oto_count = 0
        if self.cookie_domain is None:
            self.get_cookie_domain(request)

        if self.cookie_str in request.COOKIES:
            try:
                oto_count = int(request.COOKIES[self.cookie_str])
            except ValueError:
                pass

        if oto_count > (len(self.oto_templates) - 1):
            # Been through all the OTO's, send the final template
            return self.return_response(request, self.final_template)
        
        response = self.return_response(request, self.oto_templates[oto_count])
        response.set_cookie(
            self.cookie_str,
            value=(oto_count + 1),
            domain=self.cookie_domain,
            max_age=(((60 * 60) * 24) * self.cookie_days)
        )
        return response



from restlib import BaseResponse, BaseClient
from filters import BasicAuthFilter, XMLParserFilter
from helpers import url_to_base_client

# *** DEPRECATED CODE ***
from deprecated import APIError, APIClient


import copy
from django import forms
from django import template
from django.template.loader import render_to_string
from django.utils.datastructures import SortedDict

register = template.Library()


@register.filter
def quick_form(form, template='djtools/quick_form.html'):
    return render_to_string(template, {'form': form})


'''
    Adapted from http://www.djangosnippets.org/snippets/1019/
    
    Example Usage:
    
    {% load djtools_form_tags %}
    <fieldset id="contact_details">
        <legend>Contact details</legend>
        <ul>
    {% get_fieldset first_name,last_name,email,cell_phone as personal_fields from form %}
    {{ personal_fields.as_ul }}
        </ul>
    </fieldset>

    <fieldset>
        <legend>Address details</legend>
        <ul>
    {% get_fieldset street_address,post_code,city as address_fields from form %}
    {{ address_fields.as_ul }}
        </ul>
    </fieldset>
'''
class FieldSetNode(template.Node):
    def __init__(self, fields, variable_name, form):
        self.fields = fields
        self.variable_name = variable_name
        self.form = form

    def render(self, context):
        form = template.Variable(self.form).resolve(context)
        new_form = copy.copy(form)
        new_form.fields = SortedDict([
            (key, value) \
                for key, value in form.fields.items() if key in self.fields
        ])

        context[self.variable_name] = new_form
        return u''


@register.tag
def get_fieldset(parser, token):
    try:
        name, fields, as_, variable_name, from_, form = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            'bad arguments for %r' % token.split_contents()[0]
        )

    return FieldSetNode(fields.split(','), variable_name, form)



import cPickle
from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from webutils.djtools.email import send_simple_email
from webutils.datastore.models import *


def process_form(request, template, formcl, redir, map={}):
    ''' View to store all form data into a text field (pickled)
        map should be a map of form fields to request.GET fields.
        example: {'sale_id': 'oid'}
        This will set the initial value for "formcl" of the 
        field sale_id to whatever oid is set in request.GET
    '''
    if request.method == 'POST':
        form = formcl(request.POST)
        if form.is_valid():
            data = {}
            for key in form.fields.keys():
                data[key] = form.cleaned_data.get(key, '')
            dump = cPickle.dumps(data)
            ds = DataStore.objects.create(dump=dump)
            return HttpResponseRedirect(redir)
    else:
        init = {}
        for key in map.keys():
            init[key] = request.GET.get(map[key], '')
        form = formcl(initial=init)

    return render_to_response(template, {
        'form': form,
        }, context_instance=RequestContext(request))


def process_email_form(request, template, formcl, redir, map={}, defaults={}):
    ''' View to email all form data to an email address.
        Required form field: email_to, email_subject
        all other options are same as process_form above
    '''
    if request.method == 'POST':
        form = formcl(request.POST)
        if form.is_valid():
            data = {}
            for key in form.fields.keys():
                data[key] = form.cleaned_data.get(key, '')
            
            data['ip'] = request.META['REMOTE_ADDR']
            msg = 'The following data was submitted:\n\n'
            msg += '\n'.join(['%s: %s\n' % (k, v) for k, v in data.items()])
            send_simple_email(
                data['email_to'],
                data['email_subject'],
                {},
                msg,
                is_template_file=False,
            )
            return HttpResponseRedirect(redir)
    else:
        init = {}
        for key in map.keys():
            init[key] = request.GET.get(map[key], '')
            init.update(defaults)
        form = formcl(initial=init)

    return render_to_response(template, {
        'form': form,
        }, context_instance=RequestContext(request))



'''
    Requires Boto AWS library!
    http://code.google.com/p/boto/
'''
import boto
from boto.s3.key import Key


class S3Error(Exception):
    "Misc. S3 Service Error"
    pass


def checkbucket(view_func):
    def _checkbucket(self, *args, **kwargs):
        if self.bucket is None:
            raise S3Error('No assigned bucket. Must run connect()')
        return view_func(self, *args, **kwargs)
    return _checkbucket


class S3Proc(object):
    def __init__(self, aws_key, aws_secret_key, 
                                    bucket, default_perm='private'):
        self.aws_key = aws_key
        self.aws_secret_key = aws_secret_key
        self.bucket_name = bucket
        self.conn = None
        self.bucket = None
        self.perm_tuple = (
            'private',
            'public-read',
            'public-read-write',
            'authenticated-read',
        )
        if default_perm not in self.perm_tuple:
            default_perm = 'private'
        self.default_perm = default_perm
    
    def _get_perm(self, perm=None):
        if perm is None or (perm is not None and perm not in self.perm_tuple):
            perm = self.default_perm
        return perm

    def connect(self):
        self.conn = boto.connect_s3(self.aws_key, self.aws_secret_key)
        try:
            self.bucket = self.conn.create_bucket(self.bucket_name)
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))
        except boto.exception.S3CreateError:
            raise S3Error('Unable to create bucket %s' % self.bucket_name)
        return True

    @checkbucket
    def put(self, filename, data, perm=None, fail_silently=True):
        perm = self._get_perm(perm)
        obj = Key(self.bucket)
        obj.key = filename
        obj.set_contents_from_string(data)
        obj.set_acl(perm)
        return True

    @checkbucket
    def put_from_file(self, filename, remote_filename=None, 
                                                perm=None, fail_silently=True):
        ''' Upload file from disk. If you want a different 
            remote filename, specify in remote_filename
        '''
        perm = self._get_perm(perm)
        obj = Key(self.bucket)
        obj.key = remote_filename or filename
        obj.set_contents_from_filename(filename)
        obj.set_acl(perm)
        return True

    @checkbucket
    def get(self, filename):
        ''' Returns file data
        '''
        obj = Key(self.bucket)
        obj.key = filename
        try:
            return obj.get_contents_as_string()
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))

    @checkbucket
    def get_to_file(self, filename, local_filename=None):
        ''' Save file to local filename.
        '''
        obj = Key(self.bucket)
        obj.key = filename
        try:
            obj.get_contents_to_filename(local_filename or filename)
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))
        return True

    @checkbucket
    def delete(self, filename, fail_silently=True):
        ''' Returns true/false on delete 
            Raises S3Error if fail_silently is set to False
        '''
        self.bucket.delete_key(filename)
        return True

    @checkbucket
    def get_perm(self, filename):
        ''' Returns permissions for set file.
        '''
        obj = Key(self.bucket)
        obj.key = filename
        try:
            return obj.get_acl()
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))

    @checkbucket
    def set_perm(self, filename, perm, fail_silently=True):
        ''' Sets permissions on file.
        '''
        perm = self._get_perm(perm)
        obj = Key(self.bucket)
        obj.key = filename
        try:
            obj.set_acl(perm)
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))
        return True

    @checkbucket
    def set_metadata(self, filename, meta_key, meta_value):
        obj = Key(self.bucket)
        obj.key = filename
        try:
            obj.set_metadata(meta_key, meta_value)
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))
        return True

    @checkbucket
    def get_metadata(self, filename, meta_key):
        obj = Key(self.bucket)
        obj.key = filename
        try:
            return obj.get_metadata(meta_key)
        except boto.exception.S3ResponseError, err:
            raise S3Error('Error: %i: %s' % (err.status, err.reason))



from geopy import geocoders, distance
from geopy.geocoders.google import GQueryError


class GeoCode(object):
    ''' Simple class to interface with Google geocode API.
        Depends on geopy module:
        http://code.google.com/p/geopy/
    '''
    def __init__(self, api_key):
        self.api_key = api_key

    def get_geocode(self, location=None):
        ''' Function to geocode an address, zipcode, etc.
            location variable can be any of those. ie,
               90715
               Lakewood, Ca
               12345 Fake Blvd, Lakewood, Ca 90715, USA
               etc.

            returns
                (Location, (latitude, longitude))
                (u'Lakewood, Ca USA', (-32.20923, -44.20948302))
        '''
        if location is None:
            return None

        g = geocoders.Google(self.api_key)
        try:
            res = g.geocode(location, exactly_one=False)
        except GQueryError:
            return None
            
        if isinstance(res, tuple):
            return res
        else:
            # Multiple, or no, results. Grab the first one
            if res:
                res_list = list(res)
                if res_list:
                    return res_list[0]

        return None


    def parse_result(self, gtuple=None):
        ''' Function to parse a geopy google geocode result tuple
            into an address map hash
        '''
        addy_hash = {
            'country': '',
            'zipcode': '',
            'state': '',
            'city': '',
            'address': '',
            'latitude': '',
            'longitude': ''
        }
        addy_map = ['country', 'zipcode', 'state', 'city', 'address']

        if gtuple is None:
            return gtuple

        place = [x.strip() for x in gtuple[0].split(',')]
        place.reverse()
        addy_hash['latitude'] = gtuple[1][0]
        addy_hash['longitude'] = gtuple[1][1]
      
        for x in xrange(len(place)):
            if addy_map[x] == 'zipcode':
                try:
                    addy_hash[addy_map[x]] = place[x].split()[1]
                    addy_hash[addy_map[x + 1]] = place[x].split()[0]
                except IndexError:
                    # Only returns Country + Zip (or State)
                    zip_state = place[x]
                    if zip_state.isdigit():
                        addy_hash[addy_map[x]] = zip_state
                    else:
                        addy_hash[addy_map[x + 1]] = zip_state
                addy_map.pop(x + 1)
            else:
                    addy_hash[addy_map[x]] = place[x]

        return addy_hash

    def parse_location(self, address, city, state, zipcode, country):
        ''' Function to get geocode information for various 
            address setups.
            Returns hash
        '''
        addy_list = [address, city, state, zipcode, country]
        location = ', '.join([x for x in addy_list if x])

        data = self.get_geocode(location)
        if data:
            return self.parse_result(data)
        return None


    def get_distance(self, start, end):
        ''' Return the distance, in miles, between the 2 
            lat/long tuples passed to this funtion.
            Example: get_distance((24.308400, -118.3903400), (34.23420, -87.23490))
        '''
        return distance.distance(start, end).miles



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.forms import AuthenticationForm
from forms import ResetForm, PasswordChangeForm
from webutils.djtools.easyconfig import EasyConfig


class Config(object):
    ''' Base config class to easily pass forms, etc. to 
        baseacct views.
    '''
    config = EasyConfig('webutils.baseacct.Config', 'BASEACCT_CONFIG')
    
    def get_login_form(self):
        return self.config.get_object('get_login_form', AuthenticationForm)
    
    def get_login_url(self):
        # Defaults to /accounts/login/ - Ugly!
        return self.config.get_object(
            'get_login_url',
            getattr(settings, 'LOGIN_URL', '/accounts/login/'),
        )
    
    def get_reset_form(self):
        return self.config.get_object('get_reset_form', ResetForm)
    
    def get_profile_model(self):
        if not hasattr(settings, 'AUTH_PROFILE_MODULE'):
            raise ImproperlyConfigured(
                'The "AUTH_PROFILE_MODULE" setting MUST be set to use ' + \
                'this method. Please add it to your projects settings.py'
            )
        return self.config.get_object('get_profile_model', None)
    
    def get_password_change_redirect(self):
        return self.config.get_object('get_password_change_redirect', None)
    
    def get_password_change_form(self):
        return self.config.get_object(
            'get_password_change_form',
            PasswordChangeForm,
        )


import urlparse
from restlib import BaseClient
from socket import _GLOBAL_DEFAULT_TIMEOUT as default_timeout


def url_to_base_client(url, filters=[], timeout=default_timeout, debug=False):
    ''' Takes a full URL and returns a dict with appropriate 
        data, including a "ready" BaseClient instance.
    '''
    url_scheme = urlparse.urlparse(url)
    host = url_scheme.netloc
    port = url_scheme.port
    is_secure = (url_scheme.scheme == 'https')
    bc = BaseClient(
        host,
        port,
        filters=filters,
        timeout=timeout,
        is_secure=is_secure,
        debug=debug,
    )
    res = {}
    for field in url_scheme._fields:
        res[field] = getattr(url_scheme, field)
        if field == 'path' and res[field] == '':
            res[field] = '/'
    bc.url_data = res
    return bc


'''
    Requires Boto AWS library!
    http://code.google.com/p/boto/
'''
import boto

try:
    import cPickle as pickle
except ImportError:
    import pickle
    
from boto.sqs.message import Message


class SQSError(Exception):
    "Misc. S3 Service Error"
    pass


def checkqueue(view_func):
    def _checkqueue(self, *args, **kwargs):
        if self.queue is None:
            raise SQSError('No assigned queue. Must run connect()')
        return view_func(self, *args, **kwargs)
    return _checkqueue


class PickleMessage(Message):
    ''' Simple class to dump/load data into Pythons 
        pickle format upon reading or writing the 
        message body.
        
        Must use set_body and get_body methods to 
        access the data appropriately
        
        XXX: Because Message is not of an object 
        type, the user of super() is not allowed.
        Filed bug report:
        
        http://code.google.com/p/boto/issues/detail?id=311
    '''
    def set_body(self, body):
        new_body = pickle.dumps(body)
        #super(PickleMessage, self).set_body(new_body)
        Message.set_body(self, new_body)
    
    def get_body(self):
        #new_body = super(PickleMessage, self).get_body()
        new_body = Message.get_body(self)
        return pickle.loads(new_body)


class SQSProc(object):
    def __init__(self, aws_key, aws_secret_key, queue):
        self.aws_key = aws_key
        self.aws_secret_key = aws_secret_key
        self.queue_name = queue
        self.msg = Message
        self.conn = None
        self.queue = None

    def connect(self):
        self.conn = boto.connect_sqs(self.aws_key, self.aws_secret_key)
        self.queue = self.set_queue(self.queue_name)
        return True
    
    @checkqueue
    def get_all_queues(self):
        return self.conn.get_all_queues()
    
    def set_queue(self, queue_name):
        try:
            queue = self.conn.create_queue(queue_name)
            if self.queue_name != queue_name:
                self.queue_name = queue_name
            return queue
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def set_message_class(self, new_cls):
        self.msg = new_cls
        self.queue.set_message_class(new_cls)
    
    @checkqueue
    def write(self, data):
        m = self.msg()
        m.set_body(data)
        try:
            return self.queue.write(m)
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def read(self, timeout=30):
        try:
            return self.queue.read(timeout)
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def get_messages(self, count=1, timeout=30):
        try:
            return self.queue.get_messages(count, visibility_timeout=timeout)
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def delete(self, msg):
        try:
            return self.queue.delete_message(msg)
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def delete_queue(self):
        try:
            self.conn.delete_queue(self.queue)
            self.queue = None
            return True
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))
    
    @checkqueue
    def count(self):
        try:
            return self.queue.count()
        except boto.exception.SQSError, err:
            raise SQSError('Error: %i: %s' % (err.status, err.reason))



from django.conf import settings


class AddIPAddress(object):
    def process_request(self, request):
        request.ip_address = self.get_real_ip(request.META)
        return None

    def get_real_ip(self, meta_data):
        checks = ('HTTP_X_REAL_IP', 'HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR',)
        ip = None
        for check in checks:
            if check in meta_data:
                ip = meta_data[check]
                break
        return ip or ''



from django.conf import settings
from webutils.helpers import grab_from_import

def import_setting_name(setting_name, default_obj=None):
    ''' Used to import an item from a value in the current 
        Django projects settings.py
        
        Example in settings.py:
        
            FORM_TO_IMPORT = 'myproject.myapp.forms.CustomForm'
        
        Then to grab this, just call import_setting_name:
        
            form = import_setting_name('FORM_TO_IMPORT')
        
        Pass 'default_obj' 
    '''
    if hasattr(settings, setting_name):
        try:
            return grab_from_import(
                getattr(settings, setting_name),
                as_from=True,
            )
        except ImportError, err:
            if default_obj is None:
                raise ImportError(str(err))
    return default_obj


import urllib
import httplib
from filters import Filters
from socket import _GLOBAL_DEFAULT_TIMEOUT as default_timeout
from webutils.helpers import encode_dict


class BaseResponse(object):
    ''' Base HTTP Response Object '''
    def __init__(self, response, data):
        self.response = response
        self.response_data = data
        self.status = response.status
        self.reason = response.reason
        self.headers = dict([k.lower(), v] for k, v in response.getheaders())


class BaseClient(object):
    ''' Very basic REST client.
    '''
    def __init__(self, host, port=None, filters=[],
                 timeout=default_timeout, is_secure=True, debug=False):
        self.host = host
        self.filters = Filters(filters)
        self.timeout = timeout
        self.is_secure = is_secure
        self.debug = debug
        self.headers = {}
        self.request = None
        self.response = None
        if port is not None:
            self.port = port
        else:
            self.port = httplib.HTTPS_PORT if is_secure else httplib.HTTP_PORT

    def send_request(self, method='GET', path='/',
                     request=None, headers={}, opts={}, check_func=None):
        ''' Construct and send request to remote host.
        '''
        self.request = request

        # Apply any filters before request
        self.filters.apply(self)

        if isinstance(self.request, dict):
            self.request = urllib.urlencode(encode_dict(self.request))
        
        if isinstance(headers, dict):
            self.headers.update(headers)

        if opts:
            path += '?%s' % urlencode(opts)
        
        conn_cls = getattr(
            httplib,
            'HTTPSConnection' if self.is_secure else 'HTTPConnection',
        )
        conn = conn_cls(self.host, self.port, timeout=self.timeout)

        if self.debug:
            print 'DEBUG (method): ', method
            print 'DEBUG (path): ', path
            print 'DEBUG (headers): ', self.headers
            print 'DEBUG (request): ', request

        conn.request(method, path, self.request, self.headers)
        res = conn.getresponse()
        data = res.read()
        conn.close()
        
        self.response = BaseResponse(res, data)

        if self.debug:
            print 'DEBUG (response data): ', self.response.response_data
            print 'DEBUG (response headers): ', self.response.headers
            print 'DEBUG (response code): %i - %s' % \
                                (self.response.status, self.response.reason)
        
        # Apply any filters after the request
        self.filters.apply(self, 'post_request')
        
        if check_func is not None and callable(check_func):
            return check_func(self)
        return self.response
    
    def get(self, path='/', **kwargs):
        ''' Shortcut for GET request '''
        return self.send_request(method='GET', path=path, **kwargs)
    
    def delete(self, path='/', **kwargs):
        ''' Shortcut for DELETE request '''
        return self.send_request(method='DELETE', path=path, **kwargs)
    
    def post(self, request, path='/', **kwargs):
        ''' Shortcut for POST request '''
        return self.send_request(
            method='POST',
            path=path,
            request=request,
            **kwargs
        )
    
    def update(self, request, path='/', **kwargs):
        ''' Shortcut for UPDATE request '''
        return self.send_request(
            method='UPDATE',
            path=path,
            request=request,
            **kwargs
        )



''' Class to handle the cleanup of files older than X days.
    Useful for deleting old backups, etc.
'''
import os
import datetime
from webutils.cleanup.callback import CleanupCallback


class CleanupError(Exception):
    pass


class Cleanup(object):
    ''' Cleanup Class
    '''
    def __init__(self, path, days=30, delete=False, 
                                    callback=None, callback_data=None):
        if not os.path.exists(path):
            raise CleanupError('Invalid path given: %s' % path)
        
        if not os.path.isdir(path):
            raise CleanupError('Given path (%s) is not a directory' % path)
        
        self.path = path
        self.days = days
        self.delete = delete
        
        if callback is not None:
            callback_data = callback_data or {}
            callback_data.update({
                'path': self.path,
                'days': self.days,
                'delete': self.delete,
            })
            self.callback = callback(**callback_data)
        else:
            self.callback = None
    
    def run_cleanup(self):
        local_time = datetime.datetime.now()
        for fname in os.listdir(self.path):
            full_fname = os.path.join(self.path, fname)
            if os.path.isdir(full_fname):
                # Ignore directories... for now...
                continue

            s = os.stat(full_fname)
            f_time = datetime.datetime.fromtimestamp(s.st_ctime)
            if (local_time - f_time).days > self.days:
                # File is older than self.days allows. Call callback first.
                if self.callback is not None:
                    self.callback.handle_file(self.path, fname)
                    
                if self.delete:
                    # XXX Add try/except here to continue even if errors?
                    os.unlink(full_fname)
                else:
                    print 'File %s older than %i days' % \
                                                (full_fname, self.days)